package net.core.editor.roles.diagram.custom.repl;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.PrintWriter;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import org.eclipse.emf.common.util.URI;
import java.nio.channels.ByteChannel;
import java.nio.channels.FileChannel;
import java.util.Collection;

import net.core.editor.roles.diagram.custom.util.Bridge;

import static conexp.contrib.java.Main.*;

public class CoreEvaluator {
	CoreRunner _runner;
	
	public CoreEvaluator(){
		super();
	}
	public CoreEvaluator(CoreRunner runner){
		_runner = runner;
	}
	protected void exit(boolean err){
		if(err) {
			System.out.println("\nOops.\n");
			System.exit(0);
		} else {
			System.out.println("\nGoodbye.\n");
			System.exit(0);
		}
	}
	protected void help(){
		System.out.println("Available commands:");
		System.out.println("  " + CoreRunner.CLEAR + "    - clears all");
		System.out.println("  " + CoreRunner.LOAD + "     - loads an input file");
		System.out.println("    args: <'" + Bridge.BASE_CONTEXT + "'|'" + Bridge.ROLE_CONTEXT + "'|'" + Bridge.ROLE_DIAGRAM + "'> <path to input file>");
		System.out.println("  " + CoreRunner.PRINT + "    - prints a context");
		System.out.println("    args: <'" + CoreRunner.ALL + "'|'" + Bridge.BASE_CONTEXT + "'|'" + Bridge.ROLE_CONTEXT + "'|'" + Bridge.COMPOSITION_CONTEXT + "'|'" + Bridge.MERGE_CONTEXT + "'>");
		System.out.println("  " + CoreRunner.STORE + "    - stores one context in csv format:");
		System.out.println("    args: <'" + CoreRunner.ALL + "'|'" + Bridge.BASE_CONTEXT + "'|'" + Bridge.ROLE_CONTEXT + "'|'" + Bridge.COMPOSITION_CONTEXT + "'|'" + Bridge.MERGE_CONTEXT + "'|'" + Bridge.ROLE_DIAGRAM + "'> [<path>]");
		System.out.println("  " + CoreRunner.NEXT_BOND + " - shows the next available bond between base and role context");
		System.out.println("  " + CoreRunner.MERGINGS + " - prints the number of (proper) mergings between the given contexts");
		System.out.println("    args: ['" + Bridge.MERGE_CONTEXT + "']['" + CoreRunner.PROPER + "']");
		System.out.println("  " + CoreRunner.SET_PATH + "  - sets path to local eclipse distribution");
		System.out.println("    args: <path to eclipse>");
		System.out.println("  " + CoreRunner.GET_PATH + "     - prints path to local eclipse distribution");
		System.out.println("  " + CoreRunner.ECLIPSE + "  - starts eclipse");
		System.out.println("  " + CoreRunner.INSTALL + "  - installs role diagram editor plugin to eclipse");
		System.out.println("    args: ['" + CoreRunner.ALL + "']");
		System.out.println("  " + CoreRunner.EXIT + "     - exits the core");
	}
	protected void setPath(String[] commands) throws Exception{
		if(!checkArguments(commands, 2)) return;
		String p = " ";
		for(int i=1; i<commands.length; i++){
			p += commands[i] + " ";
		}
		if (p.startsWith(" ")) p = p.substring(1, p.length());
		if (p.endsWith(" ")) p = p.substring(0, p.length()-1);
		_runner._eclipsePath = URI.createFileURI(p);
		PrintWriter pw = new PrintWriter(new FileWriter(".path"));
		pw.print(p);
		pw.flush();
		pw.close();
	} 
	protected void getPath(){
		String p = decodePath(_runner._eclipsePath.devicePath());		
		System.out.println("  " + p); 
	}
	protected void runEclipse() throws Exception{
		String os = System.getProperty("os.name").toLowerCase();
		String file = "";
		if(os.indexOf("win")>=0) file = "eclipse.exe";
		else if(os.indexOf("linux")>=0) file = "eclipse";
		else if(os.indexOf("mac")>=0) file = "eclipse";
		else System.out.println("  Your OS is not supported.");
		String p = decodePath(_runner._eclipsePath.devicePath() + "/" + file);
		System.out.println("  Starting " + p);
		Runtime.getRuntime().exec(p);
	}
	protected void installEditor(String[] commands) throws Exception{
		boolean all = commands.length>1 && commands[1].equals(CoreRunner.ALL);
		//install features
		System.out.println("  Installing features.");
		File features = new File("features");
		//create destination folder
		File nFeatures = new File(decodePath(_runner._eclipsePath.devicePath() + "/features"));		
		if(!nFeatures.exists()) nFeatures.mkdir();
		transfer(features, nFeatures, all);
		//
		//install plugins
		System.out.println("  Installing plugins.");
		File plugins = new File("plugins");
		//create destination folder
		File nPlugins = new File(decodePath(_runner._eclipsePath.devicePath() + "/plugins"));
		nPlugins.mkdir();
		transfer(plugins, nPlugins, all);
	}
	protected void print(String[] commands){
		if(!checkArguments(commands, 2)) return;
		//
		String what = commands[1];
		if (what.equals(CoreRunner.ALL)) _runner._bridge.printContexts();
		else if (validArgument("print", what)) _runner._bridge.printContext(what);
		else System.out.println("  No valid argument. Type '" + CoreRunner.HELP + "' for more information");
	}
	protected void store(String[] commands){
		if(!checkArguments(commands, 2)) return;
		//
		String what = commands[1];
		String path = "";
		if(commands.length>2) path = commands[2];
		else path = Integer.toString(_runner._count);
		_runner._count++;
		if (what.equals(CoreRunner.ALL)) _runner._bridge.storeFiles(Integer.toString(_runner._count));
		else if (validArgument("store", what)) _runner._bridge.storeFile(what, path);
		else System.out.println("  No valid argument. Type '" + CoreRunner.HELP + "' for more information");
	}
	protected void load(String[] commands){
		if(!checkArguments(commands, 3)) return;
		//
		String what = commands[1];
		if (!validArgument("load", what)) return;
		String path = commands[2];
		if (!checkFile(URI.createFileURI(path))) return;
		//
		if (what.equals(Bridge.ROLE_DIAGRAM)) {
			_runner._bridge.setDiagramPath(path);
			_runner._bridge.generateContexts();
		} else {
			_runner._bridge.setContext(what, read_context(path));
		}
	}
	protected void clear() {
		_runner._bridge.clear();
	}
	@SuppressWarnings("unchecked")
	protected void nextBond(BufferedReader reader) throws Exception {
		Object base = _runner._bridge.getContext(Bridge.BASE_CONTEXT);
		Object role = _runner._bridge.getContext(Bridge.ROLE_CONTEXT);
		if (base == null || role == null) {
			System.out.println("  Missing Context.");
			return;
		}
		
		Collection bonds = (Collection) all_bonds(Bridge.contraordinalScale(base), Bridge.contraordinalScale(role));
		Object nextBond = _runner._bridge.getContext(Bridge.ROLE_PLAY);
		while (true) {
			nextBond = Bridge.nextInCollection(bonds, nextBond, true);
			System.out.println(context_to_string(nextBond));
			System.out.println("  Keep this bond? (y/n)");
			System.out.print(CoreRunner.PRE);
			String dec = reader.readLine();
			if (dec.equals("y")) {
				break;
			} else if(dec.equals(CoreRunner.EXIT)) {
				exit(false);
			}
		}
		_runner._bridge.setContext(Bridge.ROLE_PLAY, nextBond);
	}
	protected void mergings(String[] commands){
		//proper mergings?
		boolean proper = false;
		boolean context = false;
		if (commands.length>2) {
			if (commands[1].equals(Bridge.MERGE_CONTEXT) && commands[2].equals(CoreRunner.PROPER)) {
				proper = true;
				context = true;
			} else {
				System.out.println("  Invalid argument.");
				return;
			}
		} else if (commands.length>1) {
			if (commands[1].equals(Bridge.MERGE_CONTEXT)) {
				context = true;
			} else if (commands[1].equals(CoreRunner.PROPER)) {
				proper = true;
			}
		}
		Object base = _runner._bridge.getContext(Bridge.BASE_CONTEXT);
		Object role = _runner._bridge.getContext(Bridge.ROLE_CONTEXT);
		if (base == null || role == null) {
			System.out.println("  Missing Context.");
			return;
		}
		if (context) {
			Object mergings = Bridge.mergingContext(base, role, proper);
			_runner._bridge.setContext(Bridge.MERGE_CONTEXT, mergings);
		} else {
			countMergings(base, role, proper);
		}
	}
	//auxiliary methods
	protected void notValid(){
		System.out.println("  No valid command. Type '" + CoreRunner.HELP + "' for more information.");
	}
	private boolean checkArguments(String[] commands, int n){
		if (commands.length<n) {
			System.out.println("  Argument missing.");
			return false;
		}
		return true;
	}
	private boolean checkFile(URI path) {
		boolean val = true;
		try {
			new FileReader(path.devicePath());
		} catch (Exception e) {
			System.out.println("  File not found. Try again.");
			val = false;
		}
		return val;
	}
	private boolean validArgument(String method, String arg){
		boolean val = false;
		if (method.equals("print")) {
			val = arg.equals(Bridge.BASE_CONTEXT) || 
				arg.equals(Bridge.ROLE_CONTEXT) ||
				arg.equals(Bridge.COMPOSITION_CONTEXT) ||
				arg.equals(Bridge.ROLE_PLAY) || 
				arg.equals(Bridge.MERGE_CONTEXT);
		} else if (method.equals("store")) {
			val = arg.equals(Bridge.BASE_CONTEXT) || 
				arg.equals(Bridge.ROLE_CONTEXT) ||
				arg.equals(Bridge.COMPOSITION_CONTEXT) ||
				arg.equals(Bridge.ROLE_PLAY) || 
				arg.equals(Bridge.MERGE_CONTEXT) ||
				arg.equals(Bridge.ROLE_DIAGRAM);
		}else if (method.equals("load")) {
			val = arg.equals(Bridge.BASE_CONTEXT) ||
				arg.equals(Bridge.ROLE_CONTEXT) ||
				arg.equals(Bridge.ROLE_DIAGRAM);
		}
		return val;
	}
	private void transfer(File sourceFolder, File destinationFolder, boolean all) throws Exception{
		FileInputStream fileIn;
		FileChannel inChannel;
		FileOutputStream fileOut;
		FileChannel outChannel;
		double n = sourceFolder.listFiles().length;
		double i = 0;
		double j = 0;
		for(File f : sourceFolder.listFiles()){
			i++;
			if(!all){
				if(!f.getName().contains("net.core.editor")) continue;
				j = destinationFolder.getName().contains("features") ? n : j + n/4;
				i = j;
			}
			double d = Math.round((i/n)*10000.)/100.;
			if (containsNewerFile(f, destinationFolder)) {
				System.out.println("  Skipping " + f.getName() + " - " + d + "% complete.");
				continue;
			}
			System.out.println("  Copying " + f.getName() + " - " + d + "% complete.");
			fileIn = new FileInputStream(f);
			inChannel = fileIn.getChannel();
			File fn = new File(destinationFolder.getAbsolutePath() + "/" + f.getName());
			fn.createNewFile();
			fileOut = new FileOutputStream(fn);
			outChannel = fileOut.getChannel();
			transfer(inChannel, outChannel, f.length(), false);
			fileIn.close();
		}
	}
	private void transfer(FileChannel fileChannel, ByteChannel byteChannel, long lengthInBytes, boolean verbose) throws Exception{
		long overallBytesTransfered = 0L;
		long time = -System.currentTimeMillis();
		while (overallBytesTransfered < lengthInBytes) {
			long bytesTransferred = 0L;
			bytesTransferred = fileChannel.transferTo(overallBytesTransfered, Math.min(1024*1024, lengthInBytes - overallBytesTransfered), byteChannel);
			overallBytesTransfered += bytesTransferred;
			if (verbose) {
				System.out.println("Overall bytes transfered: " + overallBytesTransfered + " progress " + (Math.round(overallBytesTransfered / ((double) lengthInBytes) * 100.0)) + "%");
			}			
		}
		time += System.currentTimeMillis();
		if (verbose) {
			System.out.println("Transfered " + overallBytesTransfered + " bytes in " + (time/1000) + " s -> " + (overallBytesTransfered / 1024.0) / (time / 1000.0) + " kBytes/s");
		}
	}
	private boolean containsNewerFile(File file, File destinationFolder){
		String fName = file.getName().substring(0, file.getName().indexOf("_"));
		String fQual = file.getName().substring(file.getName().indexOf("_")+1, file.getName().lastIndexOf("."));
		for(File f : destinationFolder.listFiles()){
			String fn = f.getName();
			if(!fn.contains(".")) continue;
			String dName = fn.contains("_") ? fn.substring(0, fn.indexOf("_")) : fn.substring(0, fn.lastIndexOf("."));
			if(fName.equals(dName)){
				if(!fn.contains("_")) return false;
				String dQual = fn.substring(fn.indexOf("_")+1, fn.lastIndexOf("."));
				if(dQual.compareTo(fQual)>=0) return true;
			}
		}
		return false;
	}
	private String decodePath(String p){
		String[] parts = p.split("%20");
		p = "";
		for(String s : parts){
			p += s + " ";
		}
		if (p.endsWith(" ")) p = p.substring(0, p.length()-1); 
		return p;
	}
	@SuppressWarnings("unchecked")
	private void countMergings(Object base, Object role, boolean proper){
		if (base == null || role == null) {
			System.out.println("  Missing Context.");
			return;
		} 
		Object mc = Bridge.mergingContext(base, role, proper);
		
		System.out.println("  Number of " + (proper ? "proper " : "") + "mergings: " + ((Collection) concepts(mc)).size());
	}
}
