package net.core.editor.roles.diagram.custom.util;

import java.util.Collection;
import java.util.HashMap;

import org.eclipse.emf.common.util.URI;

import static conexp.contrib.java.Main.*;

public class Bridge {
	//IDs zum Zugriff auf die Kontexte
	public static final String BASE_CONTEXT = "bc";
	public static final String ROLE_CONTEXT = "rc";
	public static final String COMPOSITION_CONTEXT = "cc";
	public static final String BASE_SCALE = "bs";
	public static final String ROLE_SCALE = "rs";
	public static final String ROLE_PLAY = "rp";
	public static final String MERGE_CONTEXT = "mc";
	public static final String ROLE_DIAGRAM = "rd";
	
	//HashMap, die alle erzeugten Kontexte enthaelt
	protected HashMap<String, Object> _contexts = new HashMap<String, Object>();
	private ContextGenerator _cg = null;
	private DiagramGenerator _dg = null;
	
	public Bridge(){
		super();
		_cg = new ContextGenerator(this);
		_dg = new DiagramGenerator(this);
	}
	public void clear() {
		_contexts.clear();
		_cg = new ContextGenerator(this);
		_dg = new DiagramGenerator(this);
	}
	public void printContext(String key) {
		printContext(key, _contexts.get(key));
	}
	private void printContext(String type, Object c) {
		if (c!=null) {
			System.out.println(resolveKey(type));
			System.out.println(context_to_string(c));
		}else{
			System.out.println("  Context not initialized.");
		}
	}
	public void printContexts() {
		for(String s : _contexts.keySet()) {
			if(s.equals(ROLE_SCALE) || s.equals(BASE_SCALE)) continue;				
			printContext(s, _contexts.get(s));
		}
	}
	public void storeFile(String type, String name){
		String s = "";
		try {
			Integer.parseInt(name);
			s = resolveName(type) + name;
			if(s.equals("")) return;
		} catch (NumberFormatException e) {
			s = name;
		}
		
		if (type.equals(ROLE_DIAGRAM)) {
			_dg.storeDiagram(s);
		} else {
			if (!s.equals("")) {
				write_context("conexp", _contexts.get(type), s + name + ".cex");
			}
		}
	}
	public void storeFiles(String name) {
		for (String key : _contexts.keySet()) {
			String s = resolveName(key);
			if (!s.equals("")) {
				write_context("conexp", _contexts.get(key), s + name + ".cex");
			}
		}
	}
	public void setDiagramPath(String path) {
		_cg.setPath(URI.createFileURI(path));
	}
	public void setContext(String type, Object c) {
		_contexts.put(type, c); 
	}
	public Object getContext(String type) {
		return _contexts.get(type);
	}
	public void generateContexts() {
		_cg.clear();
		_cg.execute();
	}
	public void generateDiagram() {
		_dg.clear();
		_dg.execute();
	}
	private String resolveKey(String key) {
		String s = "";
		if(key.equals(BASE_CONTEXT)) s = "Base Type Context";
		else if(key.equals(BASE_SCALE)) s = "Base Type Order";
		else if(key.equals(ROLE_CONTEXT)) s = "Role Type Context";
		else if(key.equals(ROLE_SCALE)) s = "Role Type Order";
		else if(key.equals(ROLE_PLAY)) s = "Role Play Context";
		else if(key.equals(COMPOSITION_CONTEXT)) s = "Composition Context";
		else if(key.equals(MERGE_CONTEXT)) s = "Merging Context";
		return s;
	}
	private String resolveName(String type) {
		String s = "";
		if (type.equals(BASE_CONTEXT)) s = "base_";
		else if (type.equals(ROLE_CONTEXT)) s = "role_";
		else if (type.equals(COMPOSITION_CONTEXT)) s = "comp_";
		else if (type.equals(MERGE_CONTEXT)) s = "merg_";
		else if (type.equals(ROLE_PLAY)) s = "play_";
		else if (type.equals(ROLE_DIAGRAM)) s = "diag_";
		//
		return s;
	}
	@SuppressWarnings("unchecked")
	public static int indexInCollection(Collection c, Object a) {
		int i = 0;
		for (Object o : c) {
			boolean val = false;
			if (a instanceof Collection) {
				val = ((Collection) o).containsAll((Collection) a) && ((Collection) a).containsAll((Collection) o); 
			} else {
				val = o.equals(a);
			}
			if (val) break;
			i++;
		}
		int pos = (i<c.size() ? i : -1);
		return pos; 		
	}
	@SuppressWarnings("unchecked")
	public static Object getFromCollection(Collection c, int i) {
		int k = 0;
		Object res = null;
		for (Object o : c) {
			if (k==i) {
				res = o;
				break;
			}
			k++;
		}
		return res; 
	}
	@SuppressWarnings("unchecked")
	public static Object nextInCollection(Collection c, Object actual, boolean loop) {
		int i = indexInCollection(c, actual)+1;
		if (loop && i==c.size()) i = 0;
		return getFromCollection(c, i);
	}
	public static boolean leqObj(Object c, Object g, Object h) {
		Object[] A1 = {g};
		Object[] A2 = {h};
		return (leqObj(c, A1, A2));
	}
	@SuppressWarnings("unchecked")
	public static boolean leqObj(Object c, Object[] A1, Object[] A2) {
		return (((Collection) object_derivation(c, A1)).containsAll((Collection) object_derivation(c, A2)));
	}
	@SuppressWarnings("unchecked")
	public static boolean isFull(Object c) {
		return ((Collection) incidence(c)).size()==((Collection) objects(c)).size()*((Collection) attributes(c)).size();
	}
	@SuppressWarnings("unchecked")
	public static Object contraordinalScale(Object c) {
		Collection obj = (Collection) objects(c);
		
		int[] I = new int[obj.size()*obj.size()];
		
		for (int i=0; i<obj.size(); i++) {
			for (int j=0; j<obj.size(); j++) {
				Object g = getFromCollection(obj, i);
				Object h = getFromCollection(obj, j);
				if (!leqObj(c, g, h)) I[i*obj.size()+j] = 1;
			}
		}
		return make_context_from_matrix(obj, obj, I);
	}
	public static Object mergingContext(Object P, Object Q, boolean proper) {
		Object nge = createNgeContext(P, Q);
		Object psi = createPsiContext(P, Q, proper);
		Object full = createFullContext(P, Q);
		
		return context_subposition(context_apposition(psi, nge), context_apposition(nge, full)); 
	}
	@SuppressWarnings("unchecked")
	private static Object[] directProduct(Collection c1, Collection c2) {
		Object[] dp = new Object[c1.size()*c2.size()];

		int s = 0;
		for (Object p : c1) {
			for (Object q : c2) {
				dp[s++] = "(" + p + "," + q + ")"; 
			}
		}
		return dp;		
	}
	@SuppressWarnings("unchecked")
	public static Object createFullContext(Object P, Object Q) {
		Object[] Gp = directProduct((Collection) objects(P), (Collection) objects(Q));
		Object[] Mp = directProduct((Collection) attributes(P), (Collection) attributes(Q));
	
		int[] Ifull = new int[Gp.length*Mp.length];
		for (int i=0; i<Gp.length*Mp.length; i++) {
			Ifull[i] = 1;
		}
		return make_context_from_matrix(Gp, Mp, Ifull);
	}
	@SuppressWarnings("unchecked")
	public static Object createNgeContext(Object P, Object Q) {
		Object[] Gp = directProduct((Collection) objects(P), (Collection) objects(Q));
		Object[] Mp = directProduct((Collection) attributes(P), (Collection) attributes(Q));
	
		int[] Inge = new int[Gp.length*Mp.length];
		for (int i=0; i<Gp.length; i++) {
			String pq = (String) Gp[i]; 
			Object p1 = pq.substring(pq.indexOf("(")+1,pq.indexOf(","));
			Object q1 = pq.substring(pq.indexOf(",")+1,pq.indexOf(")"));
			for (int j=0; j<Mp.length; j++) {
				pq = (String) Mp[j]; 
				Object p2 = pq.substring(pq.indexOf("(")+1,pq.indexOf(","));
				Object q2 = pq.substring(pq.indexOf(",")+1,pq.indexOf(")"));
				if (incident_p(P, p2, p1)==null || incident_p(Q, q1, q2)==null) Inge[i*Mp.length+j] = 1;
			}
		}
		return make_context_from_matrix(Gp, Mp, Inge);
	}
	@SuppressWarnings("unchecked")
	public static Object createPsiContext(Object P, Object Q, boolean proper) {
		Object[] Gp = directProduct((Collection) objects(P), (Collection) objects(Q));
		Object[] Mp = directProduct((Collection) attributes(P), (Collection) attributes(Q));
		
		int[] Ipsi = new int[Gp.length*Mp.length];
		for (int i=0; i<Gp.length; i++) {
			String pq = (String) Gp[i]; 
			Object p1 = pq.substring(pq.indexOf("(")+1,pq.indexOf(","));
			Object q1 = pq.substring(pq.indexOf(",")+1,pq.indexOf(")"));
			for (int j=0; j<Mp.length; j++) {
				pq = (String) Mp[j]; 
				Object p2 = pq.substring(pq.indexOf("(")+1,pq.indexOf(","));
				Object q2 = pq.substring(pq.indexOf(",")+1,pq.indexOf(")"));
				boolean pre1 = incident_p(Q, q1, q2)!=null;
				boolean post1 = incident_p(P, p1, p2)!=null && (proper ? !((String) p1).equals((String) p2) : true);
				boolean pre2 = incident_p(P, p2, p1)!=null;
				boolean post2 = incident_p(Q, q2, q1)!=null && (proper ? !((String) q2).equals((String) q1) : true);
				
				boolean val1 = !pre1 || (pre1 && post1);
				boolean val2 = !pre2 || (pre2 && post2);
				if (val1 && val2) Ipsi[i*Mp.length+j] = 1; 
			}
		}
		return make_context_from_matrix(Gp, Mp, Ipsi);
	}
}
