package net.core.editor.roles.diagram.custom.util;

import java.io.FileWriter;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.Collection;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;

import org.jdom.Attribute;
import org.jdom.Document;
import org.jdom.Element;
import org.jdom.Namespace;
import org.jdom.output.Format;
import org.jdom.output.XMLOutputter;

import static conexp.contrib.java.Main.*;

public class DiagramGenerator {
	private Bridge _bridge = null;
	
	private Document _doc = null;

	public DiagramGenerator(){
		super();
	}
	public DiagramGenerator(Bridge bridge) {
		_bridge = bridge;
	}
	public void clear() {
		_doc = null;
	}
	public void execute() {
		generateModelFile();
	}
	@SuppressWarnings("unchecked")
	private void generateModelFile(){
		HashMap<Object, Integer> map = new HashMap<Object, Integer>();
		Namespace xsi = Namespace.getNamespace("xsi", "http://www.w3.org/2001/XMLSchema-instance");
		Namespace xmi = Namespace.getNamespace("xmi", "http://www.omg.org/XMI");
		Namespace roles = Namespace.getNamespace("roles", "http://net.core.editor/1.0"); 
		_doc = new Document(new Element("RoleModel", roles));
		Element root = _doc.getRootElement();
		root.setAttribute(new Attribute("version", "2.0", xmi));
		root.setNamespace(roles);
		root.addNamespaceDeclaration(xmi);
		root.addNamespaceDeclaration(xsi);
		HashMap<Object, Object> baseResolver = initResolver(_bridge.getContext(Bridge.BASE_CONTEXT));
		HashMap<Object, Object> roleResolver = initResolver(_bridge.getContext(Bridge.ROLE_CONTEXT));
		
		for(int i=0; i<2; i++){
			int count = -1;
			Object K;
			String s, t;
			Element active;
			if(i==0){
				K = _bridge.getContext(Bridge.BASE_CONTEXT);
				t = "baseTypes";
				active = root;
			}else{
				K = _bridge.getContext(Bridge.ROLE_CONTEXT);
				t = "roleTypes";
				Element collaborations = new Element("collaborations");
				collaborations.setAttribute("name", "coll");
				_doc.getRootElement().addContent(collaborations);
				active = collaborations;
			}
			if(K==null) continue;
			for(Object g : (Collection) objects(K)){
				count++;
				//map types and the respective fields 
				map.put(g, count);
				Element types = new Element(t);
				types.setAttribute("name", (String) g);
				for (Object m : (Collection) attributes(K)) {
					if (incident_p(K, g, m)==null) continue;
					String n = (String) m;
					if (!n.contains(":")) {
						n += ":void";
					}
					if (n.contains("(") && n.contains(")")) {
						Element methods = new Element("methods");
						String[] met = new String[2];
						met[0] = n.substring(0, n.lastIndexOf(":"));
						met[1] = n.substring(n.lastIndexOf(":")+1, n.length());
						if (n.contains("()")) {
							methods.setAttribute("name", met[0].substring(0, met[0].indexOf("(")));
							methods.setAttribute("returnType", met[1]);
						} else {
							String arguments = met[0].substring(met[0].indexOf("(")+1,met[0].indexOf(")"));
							met[0] = met[0].substring(0,met[0].indexOf("("));
							methods.setAttribute("name", met[0]);
							methods.setAttribute("returnType", met[1]);
							String[] args = arguments.split(",");
							for (String a : args) {
								Element arg = new Element("arguments");
								arg.setText(a);
								methods.addContent(arg);
							}
						}
						types.addContent(methods);
					} else {
						Element attributes = new Element("attributes");
						String[] att = n.split(":");
						attributes.setAttribute("name", att[0]);
						attributes.setAttribute("type", att[1]);
						types.addContent(attributes);
					}
				}
				active.addContent(types);
			}
			for(Object g : (Collection) objects(K)){
				for(Object h : (Collection) objects(K)){
					if (g.equals(h)) continue;
					if (isUpperNeighbor(K, g, h, (i==0) ? baseResolver : roleResolver)) {
						Element inherit = new Element("edges");
						s = (i==0 ? "BaseInheritanceEdge" : "RoleInheritanceEdge");
						inherit.setAttribute(new Attribute("type", "roles:" + s, xsi));
						inherit.setAttribute(new Attribute("source", "//@" + (i==0 ? "" : "collaborations.0/@") + t + "." + map.get(g)));
						inherit.setAttribute(new Attribute("target", "//@" + (i==0 ? "" : "collaborations.0/@") + t + "." + map.get(h)));
						root.addContent(inherit);
						//remove inherited fields
						Element source = null;
						
						Object[] A = {h};
						Collection toRem = (Collection) object_derivation(K, A); 
						
						for (Object el : active.getChildren()) {
							if (((Element) el).getName().equals("edges")) continue;
							if (((Element) el).getAttribute("name").getValue().equals(g)) {
								source = ((Element) el);
							}
						}
						if (source!=null) {
							List<Object> l = new LinkedList<Object>();
							//deep copy of source.getChildren()
							for(Object at : source.getChildren()){
								l.add(at);
							}
							for (Object at : l) {
								for (Object bt : toRem) {
									if (((Element) at).getAttribute("name").getValue().equals(bt)) {
										source.removeContent((Element) at);
									}
								}
							}
						}
					}
				}
			}
		}
		Object rp = _bridge.getContext(Bridge.ROLE_PLAY);
		if (rp!=null) {
			for (Object b : (Collection) objects(rp)) {
				for (Object r : (Collection) attributes(rp)) {
					if (incident_p(rp, b, r)==null) continue;
					//throw away implicit role play
					boolean cont = false;
					for (Object h : upperNeighbors(_bridge.getContext(Bridge.BASE_CONTEXT), b, baseResolver, true)) {
						if (incident_p(rp, (String) h, (String) r)!=null) {
							cont = true;
							break;
						}
					}
					if (cont) continue;
					for (Object h : upperNeighbors(_bridge.getContext(Bridge.ROLE_CONTEXT), r, roleResolver, false)) {
						if (incident_p(rp, (String) b, (String) h)!=null) {
							cont = true;
							break;
						}
					}
					if (cont) continue;
					//keep explicit role play
					Element rolePlay = new Element("edges");
					rolePlay.setAttribute(new Attribute("type", "roles:RolePlayEdge", xsi));
					rolePlay.setAttribute(new Attribute("source", "//@collaborations.0/@roleTypes." + map.get(r)));
					rolePlay.setAttribute(new Attribute("target", "//@baseTypes." + map.get(b)));
					root.addContent(rolePlay);
				}
			}
		}
	}
	@SuppressWarnings("unchecked")
	private Collection upperNeighbors(Object K, Object g, HashMap<Object, Object> resolver, boolean asLower) {
		Object yg = getConcept(K, g, resolver);
		Collection unc = (Collection) (asLower ? lattice_lower_neighbours(concept_lattice(K), yg) : lattice_upper_neighbours(concept_lattice(K), yg));
		Collection un = new LinkedList();
		for (Object o : unc) {
			un.add(resolver.get(o));
		}
		return un;
	}
	@SuppressWarnings("unchecked")
	private boolean isUpperNeighbor(Object K, Object g, Object h, HashMap<Object, Object> resolver) {
		//checks, if yg is covered by yh in B(K)
		Object yg = getConcept(K, g, resolver);
		boolean val = false;
		
		Collection un = (Collection) lattice_upper_neighbours(concept_lattice(K), yg);
		for (Object yh : un) {
			if (resolver.get(yh)==null) continue;
			if (resolver.get(yh).equals(h)) {
				val = true;
				break;
			}
		}
		return val;
	}
	private Object getConcept(Object K, Object g, HashMap<Object, Object> resolver) {
		Object yg = null;
		for (Object key : resolver.keySet()) {
			Object o = resolver.get(key);
			if (o.equals(g)) {
				yg = key;
				break;
			}
		}
		return yg;
	}
	@SuppressWarnings("unchecked")
	private HashMap<Object, Object> initResolver(Object K) {
		HashMap<Object, Object> resolver = new HashMap<Object, Object>();
		
		for (Object o : (Collection) concepts(K)) {
			Collection ext = (Collection) Bridge.getFromCollection((Collection) o, 0);
			for (Object g : (Collection) objects(K)) {
				Object[] A = {g};
				Collection gII = (Collection) attribute_derivation(K, object_derivation(K, A));
				if (gII.containsAll(ext) && ext.containsAll(gII)) {
					resolver.put(o, g);
					break;
				}
			}
		}
		return resolver;
	}
	
	@SuppressWarnings("unchecked")
	private HashMap<Object, Object> inferInheritance(Object K, boolean base) {
		HashMap<Object, Object> inheritance = new HashMap<Object, Object>();
		for(Object g : (Collection) objects(K)){
			for(Object h : (Collection) objects(K)){
				if(g.equals(h)) continue;
				if (Bridge.leqObj(K, h, g)) {
					if (base) inheritance.put(h, g);
					else inheritance.put(g, h);
				}
			}
		}
		return inheritance;
	}
	public void storeDiagram(String name){
		if(_doc==null) generateModelFile();
		try {
			if(!name.contains(".") || !name.substring(name.lastIndexOf(".")+1, name.length()).equals("rd")){
				name += ".rd";
			}
			XMLOutputter out = new XMLOutputter(Format.getPrettyFormat());
			PrintWriter pw = new PrintWriter(new FileWriter(name));
			out.output(_doc, pw);
			pw.close();
		} catch (IOException e) {
			e.printStackTrace();
		}
	}
}